/*
	elf_format.h
	copyright 1996-1997 by Ben Martz
	all rights reserved world wide

	ANY AND ALL MODIFICATIONS TO THIS SOURCE MUST CREDIT THE ORIGINAL
	AUTHOR, BEN MARTZ (benmartz@ic.net), AND MUST BE GIVEN TO THE AUTHOR
	FOR INTEGRATION INTO THE MAIN PowerOS SOURCE TREE. THANK YOU FOR YOUR
	COOPERATION!

	created based on the document "Executable and Linkable Format (ELF)"
	found lying around somewhere on the world wide web
*/

/* Elf32 data types */
typedef	UInt32	Elf32_Addr;
typedef	UInt16	Elf32_Half;
typedef	UInt32	Elf32_Off;
typedef	SInt32	Elf32_Sword;
typedef	UInt32	Elf32_Word;

/* Elf32 header struct */
typedef struct {
	unsigned char	ei_magic[4];
	unsigned char	ei_class;
	unsigned char	ei_data;
	unsigned char	ei_version;
	unsigned char	ei_pad[9];
	Elf32_Half		e_type;
	Elf32_Half		e_machine;
	Elf32_Word		e_version;
	Elf32_Addr		e_entry;
	Elf32_Off		e_phoff;
	Elf32_Off		e_shoff;
	Elf32_Word		e_flags;
	Elf32_Half		e_ehsize;
	Elf32_Half		e_phentsize;
	Elf32_Half		e_phnum;
	Elf32_Half		e_shentsize;
	Elf32_Half		e_shnum;
	Elf32_Half		e_shstrndx;
} Elf32_Hdr;

/* Elf32 file types */
enum {
	ET_NONE		= 0,
	ET_REL,
	ET_EXEC,
	ET_DYN,
	ET_CORE,
	ET_LOPROC	= 0xff00,
	ET_HIPROC	= 0xffff
} Elf32_Hdr_Types;

/* Elf32 machine types */
enum {
	EM_NONE		= 0,
	EM_M32,
	EM_SPARC,
	EM_386,
	EM_68K,
	EM_88K,
	EM_860,
	EM_MIPS,
	EM_PPC601	= 20	// i got this by inference - i ran tests with a pmac mklinux binary
} Elf32_Hdr_Machines;

/* Elf32 section header struct */
typedef struct {
	Elf32_Word	sh_name;
	Elf32_Word	sh_type;
	Elf32_Word	sh_flags;
	Elf32_Addr	sh_addr;
	Elf32_Off	sh_offset;
	Elf32_Word	sh_size;
	Elf32_Word	sh_link;
	Elf32_Word	sh_info;
	Elf32_Word	sh_addralign;
	Elf32_Word	sh_entsize;
} Elf32_Shdr;

/* Elf32 section types */
enum {
	SHT_NULL	= 0,
	SHT_PROGBITS,
	SHT_SYMTAB,
	SHT_STRTAB,
	SHT_RELA,
	SHT_HASH,
	SHT_DYNAMIC,
	SHT_NOTE,
	SHT_NOBITS,
	SHT_REL,
	SHT_SHLIB,
	SHT_DYNSYM,
	SHT_LOPROC	= 0x70000000,
	SHT_HIPROC	= 0x7fffffff,
	SHT_LOUSER	= 0x80000000,
	SHT_HIUSER	= 0xffffffff
} Elf32_Shdr_Types;

/* Elf32 section flag masks */
enum {
	SHF_WRITE		= 0x01,
	SHF_ALLOC		= 0x02,
	SHF_EXECINSTR	= 0x04,
	SHF_MASKPROC	= 0xf0000000
} Elf32_Shdr_Flags;

/* Elf32 loader section types */
enum {
	SECT_TEXT	= SHF_ALLOC | SHF_EXECINSTR,
	SECT_DATA	= SHF_ALLOC | SHF_WRITE,
	SECT_RODATA	= SHF_ALLOC,
	SECT_BSS	= SHT_NOBITS
} Elf32_Loader_Section_Types;